//-------------------------------------------------------------------------------------------------------------------------------
{Map Form Active/Inactive}
procedure MapFormInh(Active: Boolean);
var
  cond : Boolean;
begin
  MapForm_btn_Refresh       .Enabled := Active;
  MapForm_btn_Close         .Enabled := Active;
  MapForm_lb_Map            .Enabled := Active;
  MapForm_lb_MapType        .Enabled := Active;
  MapForm_lbl_Disc          .Enabled := Active;
  MapForm_ed_AD_Disc        .Enabled := Active;
  MapForm_ed_OziPath        .Enabled := Active;
  MapForm_ed_MobacPath      .Enabled := Active;
  MapForm_ed_GeokukPath     .Enabled := Active;
  MapForm_btn_SelDisc       .Enabled := Active;
  MapForm_btn_SelOziPath    .Enabled := Active;
  MapForm_btn_SelMobacPath  .Enabled := Active;
  MapForm_cb_Geokuk_Del     .Enabled := Active;
  MapForm_cb_ShowInfo       .Enabled := Active;
  cond := Active and (MapForm_lb_Map.ItemIndex > -1) and
         (((MapForm_lb_MapType.SelCount  >  0) and     MapForm_lb_MapType.Multiselect) OR
          ((MapForm_lb_MapType.ItemIndex > -1) and not MapForm_lb_MapType.Multiselect));    //povoli tlacitka pouze kdyz je vybrany nejaky typ mapy
  if cond then begin
    if RegexFind('(g|G)oogle', MapForm_lb_Map.Items[MapForm_lb_Map.ItemIndex]) then begin 
      MapForm_btn_ImpOzi   .Enabled := cond;
      MapForm_btn_ImpMobac .Enabled := cond;
      MapForm_btn_ImpGeokuk.Enabled := False;
    end
    else begin
      MapForm_btn_ImpOzi   .Enabled := False;
      MapForm_btn_ImpMobac .Enabled := False;
      MapForm_btn_ImpGeokuk.Enabled := cond;
    end;
    MapForm_btn_Manage   .Enabled := cond;
    MapForm_btn_Delete   .Enabled := cond;
  end
  else begin
    MapForm_btn_ImpOzi   .Enabled := False;
    MapForm_btn_ImpMobac .Enabled := False;
    MapForm_btn_ImpGeokuk.Enabled := False;
    MapForm_btn_Manage   .Enabled := False;
    MapForm_btn_Delete   .Enabled := False;
  end;
end;
//-------------------------------------------------------------------------------------------------------------------------------
{MapMng Form Active/Inactive}
procedure MapMngFormInh(Active: Boolean);
begin
  MapMngForm_btn_Refresh    .Enabled := Active;
  MapMngForm_cb_Polygons    .Enabled := Active;
  MapMngForm_lb_Polygons    .Enabled := Active;
  MapMngForm_btn_Delete     .Enabled := Active;
  MapMngForm_btn_Cancel     .Enabled := Active;
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Get coordinate from name of file}
procedure GetTilesFromFileName(MapType, FileName : String; var x, y, z, s : String);
var  
  script : String;
begin
  case MapType of                                                                                                                   
    'mapycz' : begin                                                                                                                
      script := RegexExtract('\w+\.kachle$', FileName);                                                                             
      x :=      IntToStr(HexToDec(RegExSubstitute('_[0-9]{1,2}_([0-9a-f]{7}?)_[0-9a-f]{7}\.kachle',    script, '$1')));             
      y :=      IntToStr(HexToDec(RegExSubstitute('_[0-9]{1,2}_[0-9a-f]{7}_([0-9a-f]{7}?)\.kachle',    script, '$1')));             
      z :=                        RegExSubstitute('_([0-9]{1,2}?)_[0-9a-f]{7}_[0-9a-f]{7}\.kachle',    script, '$1');             
      s :=                        RegExSubstitute('^(.*?)_[0-9]{1,2}_[0-9a-f]{7}_[0-9a-f]{7}\.kachle', script, '$1');               
    end;                                                                                                                            
    else begin                                                                                                                      
      script := RegexExtract('\d+\\\d+\\\d+(\.(png|jpg))$', FileName);                                                              
      x :=                        RegExSubstitute('\\(.*?)\\',                  script, '$1');                                      
      y :=                        RegExSubstitute('\d+\\\d+\\(.*?)\.(png|jpg)', script, '$1');                                      
      z :=                        RegexExtract('^\d+',                          script);                                          
      s := '0';                                                                                                                     
    end;                                                                                                                            
  end;                                                                                                                              
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Get name of mapycz database}
function GetFileNameMapyCz():String;
begin
  Result := RegExSubstitute('\|(.*?)\|', MapForm_lb_Map.Items[MapForm_lb_Map.ItemIndex], '$1'); 
  if (Result = '') then Result := 'mapy_cz';
end;         
//-------------------------------------------------------------------------------------------------------------------------------
{Convert tile numbers to coordinates}
procedure OSM_TilesToCoord(xtile, ytile, zoom : String; var Lat, Lon : Extended);
var
  x,y,z : Integer;
  n     : Extended;
begin
  x := StrToInt(xtile);
  y := StrToInt(ytile);
  z := StrToInt(zoom);
  n := Power(2, z);
  lat := arctan2(sinh(Pi * (1 - 2 * y / n)), 1) * 180.0 / Pi;
  lon := (x / n) * 360.0 - 180.0;
  lat := lat -(170.1/(2*n));             // shift coordinates to the centre of the tile
  lon := lon +(360/(2*n));
end;         
//-------------------------------------------------------------------------------------------------------------------------------
{Convert tile numbers to coordinates}
procedure MCZ_TilesToCoord(xtile, ytile, zoom : String; var Lat, Lon : Extended);
var
  x,y,z       : Integer;
  x_UTM,y_UTM : Extended;
  n           : Extended;
begin
  x := StrToInt(xtile);
  y := StrToInt(ytile);
  z := StrToInt(zoom);
  n := Power(2, (24-z)) / 2;
  x_UTM := (x * 0.03125) - 3700000 + n;      // UTM coordinates of the map left bottom corner 
  y_UTM := (y * 0.03125) + 1300000 + n;      // shift coordinates to the centre of the tile
  UTMtoWGS(y_UTM, x_UTM, 33, 'U', lat, lon);
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Select corresponding polygon for map tile}
procedure MapPolySelect(MapType,xtile,ytile,zoom: String);
var
  Polygons           : TGeoPolyList;
  str                : String;
  i                  : Integer;
  Lat,Lon            : Extended;
begin
  Polygons := TGeoPolyList.create;
  try
    if MapType = 'mapycz' then MCZ_TilesToCoord(xtile,ytile,zoom,Lat,Lon)
                          else OSM_TilesToCoord(xtile,ytile,zoom,Lat,Lon);     
    Polygons.init(MapMngForm_cb_Polygons.Text);
    str := Polygons.FindPolygon(Lat, Lon); 
    if str = '' then str := 'other regions';  
    for i := 0 to (MapMngForm_lb_Polygons.Items.Count - 1) do begin
      if RegexFind(str+'$', MapMngForm_lb_Polygons.Items[i]) then begin
        str := FixLen(IntToStr(StrToInt(RegexExtract('^\d+', MapMngForm_lb_Polygons.Items[i])) + 1), 8) + str;
        MapMngForm_lb_Polygons.Items[i] := str;
        str := '';
        i := MapMngForm_lb_Polygons.Items.Count; 
      end;
    end;
    if str <> '' then begin
      MapMngForm_lb_Polygons.Items.Add(FixLen('1', 8) + str);
    end;
  finally
    Polygons.Free;
  end;
end;
//-------------------------------------------------------------------------------------------------------------------------------
{GetMapInfo}
procedure GetMapInfo(Mode: String);
var 
  Map_db        : TSqliteDatabase; // databaze s mapou 
  Map_table     : TSqliteTable;    // tabulka mapovych kachli 
  Map_Num       : Integer;
  Map_Storage   : String;
  Map_File      : String;
  Map_Path      : String;
  Map_Type      : String;
  Files         : TStringList;
  FilesSpace    : Integer;
  i             : Integer;
  cond          : Boolean;
  xtile,ytile,zoom,s : String;
begin
  MapFormInh(False);
  MapMngFormInh(False);
  MapForm_lb_NumTiles  .Caption := '-';
  MapForm_lb_TotalSpace.Caption := '-';
  cond :=   (MapForm_lb_Map.ItemIndex > -1) AND
          (((MapForm_lb_MapType.SelCount  >  0) and     MapForm_lb_MapType.Multiselect) OR
           ((MapForm_lb_MapType.ItemIndex > -1) and not MapForm_lb_MapType.Multiselect)) ;    //povoli tlacitka pouze kdyz je vybrany nejaky typ mapy
  if cond then begin
    GB_Kind(_('Geting Map Info...'));
    MapMngForm_lb_Polygons.Items.Clear;
    MapMngForm_lb_Polygons.Items.Add('Select regions and click'); 
    MapMngForm_lb_Polygons.Items.Add('on the refresh button.'); 
    Map_Storage := RegexExtract('database|filesystem', MapForm_lb_Map.Items[MapForm_lb_Map.ItemIndex]);
    Map_File    := RegexExtract('\S+$', MapForm_lb_MapType.Items[MapForm_lb_MapType.ItemIndex]);
    Map_Path    := ToCardPath(AD_MapPath) + '\' + Map_File;

    if RegexFind('mapycz', Lowercase(MapForm_lb_Map.Items[MapForm_lb_Map.ItemIndex])) then Map_Type := 'mapycz'
                                                                                      else Map_Type := 'google';
    if Map_Storage = 'database' then begin  //DATABASE
      Map_Path := Map_Path + '.sqlitedb';
      if Map_Type = 'mapycz' then Map_Path := ReplaceString(Map_Path, Map_File, GetFileNameMapyCz);
      if FileExists(Map_Path) then begin
        Map_db := TSqliteDatabase.Create(Map_Path)
        if SQLTableExists(Map_db, 'tiles') then begin
          Map_Num := Map_db.GetTableValue('SELECT count(*) FROM tiles'); 
          MapForm_lb_NumTiles.Caption := IntToStr(Map_Num); 
          if Mode = 'regions' then begin
            MapMngForm_lb_Polygons.Items.Clear;
            MapMngForm_lb_Polygons.Items.Add(FixLen('0',8) + 'other regions');  // zalozi jako prvni aby byla na zacatku
            Map_table := Map_db.GetTable('SELECT x,y,z FROM tiles', false);  //
            try
              while not Map_table.EOF do begin
                if GB_Test then Break;  //preruseno uzivatelem
                GB_Progress(1, i ,Map_Num); i:=i+1;
                xtile := Map_table.FieldAsString(Map_table.FieldIndex['x']);
                ytile := Map_table.FieldAsString(Map_table.FieldIndex['y']);
                zoom  := Map_table.FieldAsString(Map_table.FieldIndex['z']);  
                if (xtile<>'') and (ytile<>'') and (zoom<>'') then begin
                  zoom := IntToStr(17 - StrToInt(zoom));  // obnovi OSM typ zoomu
                  MapPolySelect(Map_Type,xtile,ytile,zoom);
                end;
                Map_table.next;                   // skok na dalsi
              end;
            finally
              Map_table.free;
            end;
          end;
        end;
        Map_db.free;
        MapForm_lb_TotalSpace.Caption := IntToStr(GetFileSize(Map_Path)/1000000); 
      end;
    end;
    if Map_Storage = 'filesystem' then begin  //FILESYSTEM
      if Map_Type = 'mapycz' then Map_Path := ReplaceString(Map_Path, Map_File,'kachle');
      if DirectoryExists(Map_Path) then begin
        GB_Progress(2, 30 ,100);
        Files := TStringList.Create;
        try
          FileAllList(Map_Path, '.*', '.*', Files);  //naplni seznam vsech souboru z adresare a jeho podadresaru
          GB_Progress(2, 70 ,100);
          if Mode = 'regions' then begin
            MapMngForm_lb_Polygons.Items.Clear;
            MapMngForm_lb_Polygons.Items.Add(FixLen('0',8) + 'other regions');  // zalozi jako prvni aby byla na zacatku
          end;
          for i := Files.Count-1 downto 0 do begin
            if GB_Test then Break;  //preruseno uzivatelem
            GB_Progress(1, Files.Count-i ,Files.Count);
            FilesSpace := FilesSpace + GetFileSize(Files[i]);
            if Mode = 'regions' then begin
              GetTilesFromFileName(Map_Type, Files[i], xtile, ytile, zoom, s);
              if (xtile<>'') and (ytile<>'') and (zoom<>'') then begin 
                MapPolySelect(Map_Type,xtile,ytile,zoom);
              end;
            end;
          end;
          GB_Progress(2, 100 ,100);
          MapForm_lb_NumTiles  .Caption := IntToStr(Files.Count);       
          MapForm_lb_TotalSpace.Caption := IntToStr(FilesSpace/1000000);
        finally
          Files.Free;
        end; 
      end;
    end;
  end;
  MapMngForm_lb_NumTiles  .Caption := MapForm_lb_NumTiles.Caption;
  MapMngForm_lb_TotalSpace.Caption := MapForm_lb_TotalSpace.Caption;
  GB_Clean;
  MapForm_lb_ProgressText.Caption  := '- - -';
  MapForm_pb_ProgressBar2.Position := 0; 
  MapFormInh(True);
  MapMngFormInh(True);
end;
//-------------------------------------------------------------------------------------------------------------------------------
{procedury formulare - GetMapInfo}
procedure form_GetMapInfo(Sender: TObject);
begin
  MapForm_lb_NumTiles       .Enabled :=  MapForm_cb_ShowInfo.Checked;
  MapForm_lb_NumTilesLbl    .Enabled :=  MapForm_cb_ShowInfo.Checked;
  MapForm_lb_TotalSpace     .Enabled :=  MapForm_cb_ShowInfo.Checked;
  MapForm_lb_TotalSpaceLbl  .Enabled :=  MapForm_cb_ShowInfo.Checked;
  MapForm_lb_TotalSpaceUnit .Enabled :=  MapForm_cb_ShowInfo.Checked;
  if MapForm_cb_ShowInfo.Checked then GetMapInfo('basic');
end;
//-------------------------------------------------------------------------------------------------------------------------------
{procedury formulare - GetMapInfo}
procedure form_GetMapRegionInfo(Sender: TObject);
begin
  GetMapInfo('regions');
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Procedury formulare - zobrazeni seznamu typu map daneho zdroje}
procedure form_MapSel(Sender: TObject);
var
  MapConfig_db : TSqliteDatabase; // databaze se strukturou map (map-config.xml) 
  Map_ID       : String; 
  MapConfig    : TSqliteTable;    // tabulka map
begin
  if MapForm_lb_Map.ItemIndex > -1 then begin
    MapForm_btn_Manage    .Enabled := False;
    MapForm_btn_Delete    .Enabled := False;
    MapMngForm_lb_TotalSpace.Caption := '-'; MapForm_lb_TotalSpace   .Caption := '-';
    MapMngForm_lb_NumTiles  .Caption := '-'; MapForm_lb_NumTiles     .Caption := '-';
    MapForm_btn_ImpOzi    .Enabled := False;
    MapForm_btn_ImpMobac  .Enabled := False;
    MapForm_btn_ImpGeokuk .Enabled := False;
    with MapForm_lb_MapType do begin
      Items.Clear;
      ItemIndex := -1;
      MapConfig_db := TSqliteDatabase.Create(MakroPath + 'Data\map-config.db3');
      try
        MultiSelect := RegexFind('(M|m)apy(C|c)z', MapForm_lb_Map.Items[MapForm_lb_Map.ItemIndex]); 
        Map_ID := RegexExtract('\d+$', MapForm_lb_Map.Items[MapForm_lb_Map.ItemIndex]);
        if SQLTableExists(MapConfig_db, 'map_type') then begin
          MapConfig := MapConfig_db.GetTable('SELECT * FROM map_type WHERE key="' + Map_ID + '" ORDER BY name ASC' , false);  //
          try
            while not MapConfig.EOF do begin
              Items.Add(FixLen(UtfToAnsi(MapConfig.FieldAsString(MapConfig.FieldIndex['name'])),17)+TAB+FixLen(MapConfig.FieldAsString(MapConfig.FieldIndex['mode']),8)+TAB+MapConfig.FieldAsString(MapConfig.FieldIndex['file']));
              MapConfig.next;                   // skok na dalsi
            end;
          finally
            MapConfig.free;
          end;
        end;
      finally
        MapConfig_db.free;
      end;
    end;
  end;
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Procedury formulare - aktivuje prislusna tlacitka}
procedure form_MapTypeSel(Sender: TObject);
begin
  MapFormInh(True);  // aktivuje prislusna tlacitka a funkce
  if MapForm_cb_ShowInfo.Checked then GetMapInfo('basic');      //pocet kachli a mnozstvi dat
end;
//===============================================================================================================================
//-------------------------------------------------------------------------------------------------------------------------------
{Read map-config.xml}      //nacte do databaze seznam definovanych map v map-config.xml
procedure ReadMapConfig(MapConfigFileName: String);
var  
  MapConfig_db                          : TSqliteDatabase; // databaze seznamu map 
  xml                                   : String;
  MapSource, MapSourceCache             : String;
  mskey,mstype,msname,msstorage, msfile : String;
  MapType, MapTypeCache                 : String;
  mtname,mtfile,mtmode                  : String;
  j                                     : integer;
begin
  if FileExists(MapConfigFileName) then begin 
    xml := FileToString(MapConfigFileName);
    MapConfig_db := TSqliteDatabase.Create(MakroPath + 'Data\Map-config.db3');
    try
      j := 0;
      repeat
        if GB_Test then Break; //preruseno uzivatelem
        j := j + 1; 
        GB_Progress(2, j-1 ,j);
        MapSource := fetch(xml, '</MapSource>'); MapSourceCache := MapSource;
        MapType := fetch(MapSource, '<MapType');   //poprve ukosnout pocatecni tagy
        mstype    := Lowercase(RegExSubstitute('<type>(.*?)</type>'      , MapType, '$1'));
        msname    :=           RegExSubstitute('<name>(.*?)</name>'      , MapType, '$1');
        msstorage := Lowercase(RegExSubstitute('<storage>(.*?)</storage>', MapType, '$1')); if msstorage = ''                                then msstorage := 'filesystem';   // default value
        msfile    := Lowercase(RegExSubstitute('<file>(.*?)</file>'      , MapType, '$1')); if msfile    = ''                                then msfile    := 'mapy_cz';      // default value
                                                                                            if (mstype<>'mapycz') or (msstorage<>'database') then msfile    := '';
        MapConfig_db.ExecSQL('BEGIN TRANSACTION');
        if (mstype <> '') and (msname <> '') then begin
          mskey := MapConfig_db.GetTableString('SELECT key FROM map_source WHERE type="'+mstype+'" AND name="'+msname+'" AND storage="'+msstorage+'" AND file="'+msfile+'"');
          if mskey = '' then begin 
            MapConfig_db.ExecSQL('INSERT INTO map_source (type,name,storage,file) VALUES ("' + mstype + '","' + msname + '","' + msstorage + '","' + msfile + '")');
            mskey := MapConfig_db.GetTableString('SELECT seq FROM sqlite_sequence WHERE name="map_source"');
          end;
          repeat
            MapType := fetch(MapSource, '</MapType>'); MapTypeCache := MapType;
            mtname  :=           RegExSubstitute('<name>(.*?)</name>', MapType, '$1');
            mtmode  := Lowercase(RegExSubstitute('mode="(.*?)">'     , MapType, '$1'));
            mtfile  := Lowercase(RegExSubstitute('<file>(.*?)</file>', MapType, '$1'));
            if (mtname <> '') and (mtfile <> '') then begin
              MapConfig_db.ExecSQL('INSERT OR REPLACE INTO map_type (key,name,file,mode) VALUES ("' + mskey + '","' + mtname + '","' + mtfile + '","' + mtmode + '")');
            end;
          until (MapTypeCache = '')            
        end;
        MapConfig_db.ExecSQL('COMMIT');
      until (MapSourceCache = '')
    finally
      MapConfig_db.free;
    end;
    GB_Progress(2, 0 ,100);
  end
  else begin
    ShowMessage(_('File not found:') + CRLF + MapConfigFileName + CRLF + CRLF +
                _('If you use Off Line mode or External disc mode, ')+
                _('you have to connect device at least once by SD Card mode before ')+
                _('and then upload settings.'));
  end;
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Read Maps from map-config.db3}
procedure form_MapRead(Sender: TObject);
var  
  MapConfig_db        : TSqliteDatabase; // databaze seznamu map 
  Map                 : TSqliteTable;    // tabulka map
begin
  MapFormInh(False);
  GB_Progress(1, 10 ,100);
  with MapForm_lb_Map do begin
    Items.Clear;
    MapForm_lb_MapType.Items.Clear;
    Sorted := False;
    MapConfig_db := TSqliteDatabase.Create(MakroPath + 'Data\Map-config.db3');
    try
      if SQLTableExists(MapConfig_db, 'map_source') then begin
        case TBitBtn(Sender).Name of       // nacte tabulku stop dle zvoleneho razeni
          'btn_ArrDate_Dwn': Map := MapConfig_db.GetTable('SELECT * FROM map_source ORDER BY name ASC' , false);
          'btn_ArrID_Dwn'  : Map := MapConfig_db.GetTable('SELECT * FROM map_source ORDER BY key  ASC' , false);
          'btn_ArrID_Up'   : Map := MapConfig_db.GetTable('SELECT * FROM map_source ORDER BY key  DESC', false);
          else               Map := MapConfig_db.GetTable('SELECT * FROM map_source ORDER BY type ASC' , false);
        end;
        try
          while not Map.EOF do begin
            Items.Add(FixLen(UtfToAnsi(    Map.FieldAsString(Map.FieldIndex['name']     )),30) +
                      FixLen(         (    Map.FieldAsString(Map.FieldIndex['type']     )), 6) +
                      FixLen(         (    Map.FieldAsString(Map.FieldIndex['storage']  )),10) +
                      FixLen(         ('|'+Map.FieldAsString(Map.FieldIndex['file'])+'|') ,17) +
                                      (    Map.FieldAsString(Map.FieldIndex['key']      ))   );
            Map.next;                   // skok na dalsi
          end;
        finally
          Map.free;
        end;
      end
      else Items.Add('- maps table not found -');
    finally
      MapConfig_db.free;
    end;
    GB_Progress(1, 0 ,100);
  end;  
  GB_Kind(_('Map-config reading done'));
  MapFormInh(True);
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Refresh Map-config database}
procedure form_MapConfDbRefr(Sender: TObject);
var  
  MapConfig_db        : TSqliteDatabase; // databaze seznamu map 
  MapFileList         : TStringList;
  i                   : integer;
begin
  MapFormInh(False);
  GB_Kind(_('Map-config.xml is reading...'));
  GB_Progress(1, 10 ,100);
  //zalozi databazi seznamu definovanych map v map-config.xml
  if FileExists(MakroPath + 'Data\Map-config.db3') then DeleteFile(MakroPath + 'Data\Map-config.db3');
  MapConfig_db := TSqliteDatabase.Create(MakroPath + 'Data\Map-config.db3');
  try
    MapConfig_db.ExecSQL('CREATE TABLE map_source (key INTEGER PRIMARY KEY AUTOINCREMENT,type STRING,name STRING,storage STRING,file STRING)');
    MapConfig_db.ExecSQL('CREATE TABLE map_type   (key INTEGER,name STRING,file STRING,mode STRING, PRIMARY KEY(key,name,file,mode))');
  finally
    MapConfig_db.free;
  end;
  //read the orig. config. file from the aDrake install dir
  GB_Progress(1, 20 ,100);
  if FileExists(MakroPath + 'Data\map-config-' + AD_Lang + '.xml') then
    ReadMapConfig(MakroPath + 'Data\map-config-' + AD_Lang + '.xml')
  else
    ReadMapConfig(MakroPath + 'Data\map-config.xml');
  //List of map-config.xml files from the aDrake data dir
  GB_Progress(1, 30 ,100);
  MapFileList  := TStringList.Create;
  try
    FileList(ToCardPath(AD_MapPath), MapFileList);
    for i := (MapFileList.Count - 1) downto 0 do begin     //necha pouze *map-config.xml soubory
      if RegexFind('.*map-config.xml$', LowerCase(MapFileList[i])) then begin
        ReadMapConfig(MapFileList[i]);
      end;
    end;
  finally
    MapFileList.free;
  end;
  form_MapRead(MapForm_btn_Refresh);    //nacte seznam map z databaze map-config.db3
  GB_Clean;
  MapFormInh(True);
end;
//===============================================================================================================================
//===============================================================================================================================
{Kopiruje mapy PC<->PC vcetne podslozek, pouze soubory a cesta dle zadane regex masky a dle typu map}
procedure MapCopy(FromDir: string; ToDir: string; RegexMaskPath: string; RegexMaskFile: string; FileMinSize: LongInt; MapType: string; Caption: string; Storage: string);
var
  Map_db        : TSqliteDatabase; // databaze s mapou 
  Map_table     : TSqliteTable;    // databaze s mapou 
  Files         : TStringList;
  NameDestFile  : string;
  MapDestExists : boolean;
  MapDestSize   : LongInt;
  MapSrcSize    : LongInt;
  NumMapsDone   : Integer;
  n             : Integer;
  x,y,z,s       : String;
begin
  if not RegexFind('\\$',FromDir) then FromDir := FromDir + '\';
  if not RegexFind('\\$',ToDir)   then ToDir   := ToDir   + '\';
  if (Storage = 'database')       then ToDir   := RegexReplace('\\$', ToDir, '', False);   //pokud je database, pak bez lomitka, bo se tvori _ToDir_.sqlitedb
  if DirectoryExists(FromDir) then begin
    GB_Caption(Caption);
    GB_Kind(_('files are calculated:') + RegexExtract('[^\\]+\S$',FromDir) + ' "' + RegexMaskFile + '"');
    GB_Value(FromDir + RegexMaskFile);
    Files := TStringList.Create;
    try
      FileAllList(FromDir, RegexMaskPath, RegexMaskFile, Files);  //naplni seznam vsech souboru z adresare a jeho podadresaru
      GB_Kind(_('files are copied: ') + RegexExtract('[^\\]+\S$',FromDir) + ' "' + RegexMaskFile + '"');
      (*FILESYSTEM*)
      if Storage = 'filesystem' then begin
        for n := 0 to Files.Count-1 do begin
          if GB_Test then Break;  //preruseno uzivatelem
          GB_Progress(1, n ,Files.Count);
          NameDestFile  := ToDir + ReplaceString(Files[n], FromDir, '');
          MakeDirPC(ReplaceString(NameDestFile, RegexExtract('[^\\]+\S$',NameDestFile), ''));
          MapDestExists := FileExists(NameDestFile);
          MapDestSize   := GetFileSize(NameDestFile);
          MapSrcSize    := GetFileSize(Files[n]);
          if (MapSrcSize > FileMinSize) then begin
            if not MapDestExists or (MapType <> 'ozi') then begin
              CopyFile(Files[n], NameDestFile);
              NumMapsDone := NumMapsDone + 1;
            end
            else if (MapSrcSize > MapDestSize) then begin
              DeleteFile(NameDestFile);
              CopyFile(Files[n], NameDestFile);
              NumMapsDone := NumMapsDone + 1;
            end;
          end;
          if (MapType = 'mapycz') and MapForm_cb_GeoKuk_Del.Checked then DeleteFile(Files[n]);
        end;
      end
      (*DATABASE*)
      else begin
        if (MapType = 'mapycz') then NameDestFile := ReplaceString(ToDir, 'kachle', GetFileNameMapyCz)
                                else NameDestFile := ToDir;   
        MakeDirPC(ReplaceString(NameDestFile, RegexExtract('[^\\]+\S$',NameDestFile), ''));  
        Map_db := TSqliteDatabase.Create(NameDestFile + '.sqlitedb');
        try
          if not SQLTableExists(Map_db, 'info')  then Map_db.ExecSQL('CREATE TABLE info(minzoom,maxzoom)');
          if not SQLTableExists(Map_db, 'tiles') then Map_db.ExecSQL('CREATE TABLE tiles(x int, y int, z int, s int, image blob, PRIMARY KEY (x,y,z,s))');
          Map_db.ExecSQL('BEGIN TRANSACTION');
          for n := 0 to Files.Count-1 do begin
            if GB_Test then Break;  //preruseno uzivatelem
            GB_Progress(1, n ,Files.Count);
            GetTilesFromFileName(MapType, Files[n], x, y, z, s);
            if (x<>'') and (y<>'') and (z<>'') and (s<>'') then begin
              z := IntToStr(17 - StrToInt(z));  //prevod do formatu zoomu BigPlanet-RMaps SQLite databaze
              MapSrcSize  := GetFileSize(Files[n]);
              MapDestSize := 0;
              if (MapType = 'ozi') then begin  // u map z OZIexpl. se prepisuji mensi dily mapovych ctvercu vetsimi, pro odstraneni bilych okraju map 
                Map_table := Map_db.GetTable('SELECT * FROM tiles WHERE x="' + x +'" AND y="' + y + '" AND z="' + z + '" AND s="' + s + '"' , false);  //
                try
                  if not Map_table.EOF then MapDestSize := Length(Map_table.FieldAsBlobText(Map_table.FieldIndex['image']));
                finally
                  Map_table.free;
                end;
              end;
              if (MapSrcSize > FileMinSize) and (MapSrcSize > MapDestSize) then begin
                Map_db.AddParamBlobText(':pblob', FileToString(Files[n]));
                Map_db.ExecSQL('INSERT OR REPLACE INTO tiles (x,y,z,s,image) VALUES("' + x +'","' + y + '","' + z + '","' + s + '",:pblob)');      
                NumMapsDone := NumMapsDone + 1;
              end;
            end;
          end;
          Map_db.ExecSQL('COMMIT');
        finally
          Map_db.free;
        end;  
      end;
      (*XXXXXXXXX*)
      NumCopyImgDone := NumCopyImgDone + NumMapsDone; NumCopyImgAll  := NumCopyImgAll  + Files.Count;
      GB_Kind(_(' .. Copying done: ') + IntToStr(NumMapsDone) + '/' + IntToStr(Files.Count) + _(' maps ')); 
    finally
      Files.Free;
    end;
    GB_Clean;
  end;
end;
//===============================================================================================================================
//===============================================================================================================================
{Maze offline mapy vcetne podslozek, soubory dle zadaneho nazvu/masky}
procedure MapDelete(MapDir: string; MapFiles: string; MapType: string; Storage: string);
var
  Map_db        : TSqliteDatabase; // databaze s mapou 
  str           : String;
begin
  if not RegexFind('\\$',MapDir) then MapDir := MapDir + '\';
  if (Storage = 'database') then begin //####### DATABASE #######
    if (MapType = 'mapycz') then begin //======= mapycz
      MapDir := MapDir + GetFileNameMapyCz + '.sqlitedb';
      if (MapForm_lb_MapType.SelCount < MapForm_lb_MapType.Items.Count) then begin    //pokud nejsou oznaceny vsechny mapy
        Map_db := TSqliteDatabase.Create(MapDir);
        try
          if not SQLTableExists(Map_db, 'tiles') then Map_db.ExecSQL('CREATE TABLE tiles(x int, y int, z int, s int, image blob, PRIMARY KEY (x,y,z,s))');
          Map_db.ExecSQL('BEGIN TRANSACTION');
          while MapFiles <> '' do begin
            if RegexFind('|', MapFiles) then begin str := Fetch(MapFiles, '|');     end
                                        else begin str := MapFiles; MapFiles := ''; end;
            Map_db.ExecSQL('DELETE FROM tiles WHERE s=' + '"' + str + '"');   
          end;
          Map_db.ExecSQL('COMMIT');
          Map_db.ExecSQL('VACUUM');
        finally
          Map_db.free;
        end;  
      end
      else begin           //pokud jsou oznaceny vsechny mapy, pak smaz celou databazi
        if MessBox(_('Are you sure to delete file?') +CRLF+ MapDir, _('Delete files'), 4) = 6 then DeleteFile(MapDir);  
      end;
    end                                
    else begin                         //======= google
      MapDir := MapDir + MapFiles + '.sqlitedb';
      if MessBox(_('Are you sure to delete file?') +CRLF+ MapDir, _('Delete files'), 4) = 6 then DeleteFile(MapDir);  
    end;
  end
  else begin                           //####### FILESYSTEM #######           !!!!!!!!DORESIT!!!!!!!!!!
    if (MapType = 'mapycz') then begin //======= mapycz
      MapDir := MapDir + 'kachle\' + MapFiles;   //vzdy jsou v adresari "kachle"
    end
    else begin                         //======= google
      MapDir := MapDir + MapFiles;
      if MessBox(_('Are you sure to delete directory?') +CRLF+ MapDir, _('Delete directory'), 4) = 6 then DelTree(MapDir);
    end;
  end;
  if MapForm_cb_ShowInfo.Checked then GetMapInfo('basic');      //obnovi pocet kachli a mnozstvi dat
end;
//===============================================================================================================================
//---- MAPC2MAPC ----------------------------------------------------------------------------------------------------------------
//===============================================================================================================================
{kopiruje obsah atlasu vytvorenych pomoci MAPC2MAPC z OziExploreru}
procedure OziAtlCopy(FromDir: string; ToDir: string; Storage: string);
var
   fl, Atlases  : TStringList;
   n : integer;
begin
  if not RegexFind('\\$',FromDir) then FromDir := FromDir + '\';
  if not RegexFind('\\$',ToDir)   then ToDir   := ToDir   + '\';
  if DirectoryExists(FromDir) then begin
    fl       := TStringList.Create;
    Atlases  := TStringList.Create;
    if RegexFind('_atlas\\$',FromDir) then begin
      Atlases.Add(FromDir);
    end
    else begin  
      FileList(FromDir,fl);
      for n := 0 to (fl.Count-1) do if RegexFind('_atlas\\$',fl[n]) then Atlases.Add(fl[n]);
    end;
    n := Atlases.Count;
    GB_Progress(2, 1 ,n);
    NumCopyImgDone := 0; NumCopyImgAll := 0;
    while (Atlases.Count > 0) do begin
      GB_Kind(_('Copying files from: ') + RegexExtract('[^\\]+_atlas', Atlases[0]));
      GB_Progress(2, n-Atlases.Count+1 ,n);
      if GB_Test then Break; //preruseno uzivatelem
      MapCopy(Atlases[0], ToDir, '\d+\\$', '\d+\.(png|jpg)$', 2000, 'ozi', _('Offline maps are copied to: ') + ReplaceString(ToDir, AD_MapPath, '') + ' (' + IntToStr(n-Atlases.Count+1) + '/' + IntToStr(n) + ')', Storage);
      Atlases.Delete(0);
    end;
    LogWork(_('= Maps from MAPC2MAPC to PDA transferred') + '(' + IntToStr(NumCopyImgDone) + '/' + IntToStr(NumCopyImgAll) + ')');
    MapForm_lb_ProgressText.Caption := _(' .. Copying done: ') + IntToStr(NumCopyImgDone) + '/' + IntToStr(NumCopyImgAll) + _(' maps '); 
    fl.Free;
    Atlases.Free;
    GB_Clean;
  end;
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Import Maps from MAPC2MAPC}
procedure form_MapImpOzi(Sender: TObject);
var
  Map_ID       : String; 
  Map_Storage  : String; 
  Map_File     : String;
begin
  MapFormInh(False);
  if IsAdOnCard then begin
    Map_ID      := RegexExtract('\d+$',                MapForm_lb_Map    .Items[MapForm_lb_Map    .ItemIndex]);
    Map_Storage := RegexExtract('database|filesystem', MapForm_lb_Map    .Items[MapForm_lb_Map    .ItemIndex]);
    Map_File    := RegexExtract('\S+$',                MapForm_lb_MapType.Items[MapForm_lb_MapType.ItemIndex]);
    GB_Caption(_('Offline maps from MAPC2MAPC synchronization'));
    OziMapPath := MapForm_ed_OziPath.Text;
    if OziMapPath <> '' then begin
      SaveIniStrValue('DEFAULT', 'LastOziMapPath', OziMapPath);                                
      GB_Kind(_('files are copying, pleas wait...'));
      OziAtlCopy(OziMapPath, ToCardPath(AD_MapPath + '\' + Map_File), Map_Storage);
    end
    else LogWork(_('# Error: Mapos from ')+RegexExtract('[^\\]+_atlas', OziMapPath)+_(' to PDA not transferred!'));
    GB_Clean;
    MapForm_pb_ProgressBar2.Position := 0; 
  end;
  MapFormInh(True);
end;
//-------------------------------------------------------------------------------------------------------------------------------
{OziPath}
procedure form_OziPath(Sender: TObject);
var
  script : String;
begin
  script := SelectDir(OziMapPath, _('Select folder of MAPC2MAPC atlas')+CRLF+
                                  '(' + _('example:') + ' C:\MAPC2MAPC\Maps\CZ_09-29_atlas)');
  if script <> '' then begin   
    OziMapPath              := script;                                 
    MapForm_ed_OziPath.Text := script;
    SaveIniStrValue('DEFAULT', 'LastOziMapPath', OziMapPath);
  end;
end;
//===============================================================================================================================
//---- MobAtlCr -----------------------------------------------------------------------------------------------------------------
//===============================================================================================================================
{Import Maps from MobAtlCr}
procedure form_MapImpMobac(Sender: TObject);
var
  Map_ID       : String; 
  Map_Storage  : String; 
  Map_File     : String;
begin
  MapFormInh(False);
  if IsAdOnCard then begin
    Map_ID      := RegexExtract('\d+$',                MapForm_lb_Map    .Items[MapForm_lb_Map    .ItemIndex]);
    Map_Storage := RegexExtract('database|filesystem', MapForm_lb_Map    .Items[MapForm_lb_Map    .ItemIndex]);
    Map_File    := RegexExtract('\S+$',                MapForm_lb_MapType.Items[MapForm_lb_MapType.ItemIndex]);
    GB_Caption(_('Offline maps from MobileAtlasCreator synchronization'));
    MobAtlCrPath := MapForm_ed_MobacPath.Text;
    if MobAtlCrPath <> '' then begin
      GB_Kind(_('transferring maps...'));
      if DirectoryExists(MobAtlCrPath) then begin
        MapCopy(MobAtlCrPath, ToCardPath(AD_MapPath) + '\' + Map_File, '\d+\\$', '\d+\.(png|jpg)$', 0, 'mobac', _('Offline maps from MobileAtlasCreator are synchronized.'), Map_Storage);
      end;
      LogWork(_('= Maps from MobileAtlasCreator to PDA transferred'));
    end
    else LogWork(_('# Error: Mapos from MobileAtlasCreator to PDA not transferred!'));
    GB_Clean;
    MapForm_pb_ProgressBar2.Position := 0; 
  end;
  MapFormInh(True);
end;
//-------------------------------------------------------------------------------------------------------------------------------
{MobacPath}
procedure form_MobacPath(Sender: TObject);
var
  script : String;
begin
  script := SelectDir(MobAtlCrPath, _('Select folder of atlas') + ', ' + _('example:')+CRLF+
                                      'C:\MobileAtlasCreator\atlases\MyAtlas_2010-09-03_211537\Google');
  if script <> '' then begin   
    MobAtlCrPath              := script;                                 
    MapForm_ed_MobacPath.Text := script;
    SaveIniStrValue('DEFAULT', 'LastMobAtlCrPath', MobAtlCrPath);
  end;
end;
//===============================================================================================================================
//---- GeoKuk -------------------------------------------------------------------------------------------------------------------
//===============================================================================================================================
{Import Maps from GeoKuk}
procedure form_MapImpGeokuk(Sender: TObject);
var
  Map_ID       : String; 
  Map_Storage  : String; 
  Map_File     : String;
  i : Integer;
begin
  MapFormInh(False);
  if IsAdOnCard then begin
    Map_ID      := RegexExtract('\d+$',                MapForm_lb_Map.Items[MapForm_lb_Map.ItemIndex]);
    Map_Storage := RegexExtract('database|filesystem', MapForm_lb_Map.Items[MapForm_lb_Map.ItemIndex]);
    GB_Caption(_('Offline maps from GeoKuk synchronization'));
    // zjisteni cesty k mapovym kachlim programu GeoKuk
    GB_Kind(_('searching maps path...'));
    if DirectoryExists(SmartPath_GK(GeoKukPath)) then begin
      GB_Kind(_('transferring maps...'));
      NumCopyImgDone := 0; NumCopyImgAll := 0;
      for i := (MapForm_lb_MapType.Items.Count - 1) downto 0 do begin 
        if MapForm_lb_MapType.Selected[i] then begin
          if Map_File <> '' then Map_File := Map_File + '|';
          Map_File := Map_File + RegexExtract('\S+$', MapForm_lb_MapType.Items[i]);
        end;
      end;
      if Map_File <> '' then begin
        MapCopy(SmartPath_GK(GeoKukPath), ToCardPath(AD_MapPath + '\kachle'), '\\$', '(' + Map_File + ')' + '_.*\.kachle', 0, 'mapycz', _('Offline maps from GeoKuk are synchronized'), Map_Storage);
        if MapForm_cb_GeoKuk_Del.Checked then begin
          MapForm_lb_ProgressText.Caption := _('Deleting files...'); 
          DelTree(SmartPath_GK(GeoKukPath));
          MakeDirPC(SmartPath_GK(GeoKukPath));     
        end;
      end;
      MapForm_lb_ProgressText.Caption := _(' .. Copying done: ') + IntToStr(NumCopyImgDone) + '/' + IntToStr(NumCopyImgAll) + _(' maps '); 
      LogWork(_('= Maps from GeoKuk to PDA transferred') + '(' + IntToStr(NumCopyImgDone) + '/' + IntToStr(NumCopyImgAll) + ')');
    end
    else begin
      ShowMessage(_('Location of GeoKuk maps failed!')+CRLF+ SmartPath_GK(GeoKukPath));
      LogWork(_('# Error: Mapos from GeoKuk to PDA not transferred!'));
    end;
    GB_Clean;
    MapForm_pb_ProgressBar2.Position := 0; 
  end; 
  MapFormInh(True);
end;
//===============================================================================================================================
//---- Maps Delete --------------------------------------------------------------------------------------------------------------
//===============================================================================================================================
{Delete selected maps}
procedure form_MapDelete(Sender: TObject);
var
  Map_ID       : String; 
  Map_Storage  : String; 
  Map_File     : String;
  Map_Type     : String;
  i : Integer;
begin
  MapFormInh(False);
  if IsAdOnCard then begin
    Map_ID      := RegexExtract('\d+$',                MapForm_lb_Map.Items[MapForm_lb_Map.ItemIndex]);
    Map_Storage := RegexExtract('database|filesystem', MapForm_lb_Map.Items[MapForm_lb_Map.ItemIndex]);
    Map_Type    := RegexExtract('google|mapycz'      , MapForm_lb_Map.Items[MapForm_lb_Map.ItemIndex]);
    GB_Caption(_('Offline maps are deleted'));

      for i := (MapForm_lb_MapType.Items.Count - 1) downto 0 do begin           //pokud je vybrano vice map (pouze mapy_cz), pak sestavi regex masku
        if MapForm_lb_MapType.Selected[i] then begin
          if Map_File <> '' then Map_File := Map_File + '|';
          Map_File := Map_File + RegexExtract('\S+$', MapForm_lb_MapType.Items[i]);
        end;
      end;
      if Map_File <> '' then begin
        MapDelete(ToCardPath(AD_MapPath), Map_File, Map_Type, Map_Storage);
      end;
      LogWork(_('= Ofline Maps deleted'));

    GB_Clean;
    MapForm_pb_ProgressBar2.Position := 0; 
  end; 
  MapFormInh(True);
end;
//===============================================================================================================================
//===============================================================================================================================
{Stop copying}
procedure form_MapStop(Sender: TObject);
begin
  EventCancel := True;
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Show window - Maps Manager}
procedure form_ShowMapForm(Sender: TObject);
begin
  ReadADini(True, False); // Nacteni nastaveni aDrake z ini
  MapForm_ed_AD_disc.Text    := CardPath;
  MapForm_ed_OziPath.Text    := OziMapPath;
  MapForm_ed_MobacPath.Text  := MobAtlCrPath;
  MapForm_ed_GeokukPath.Text := SmartPath_GK(GeoKukPath) + '\';
  form_MapRead(MapForm_btn_Refresh);    //nacte seznam map z databaze map-config.db3
  GB_Caption(_('Maps manager showed'));
  MapForm.ShowModal;
end;
//-------------------------------------------------------------------------------------------------------------------------------
{Show window - Map Manage}
procedure form_MapManage(Sender: TObject);
var
  Polygons : TStringList;
  i        : Integer;
begin
  MapMngForm_lb_Name.Caption := RegexReplace('\s{2,}', MapForm_lb_Map.Items[MapForm_lb_Map.ItemIndex], ' / ', False);
  if RegexFind('mapycz', Lowercase(MapForm_lb_Map.Items[MapForm_lb_Map.ItemIndex])) then
    MapMngForm_lb_Type.Caption := 'All type of mapy_cz tiles'
  else
    MapMngForm_lb_Type.Caption := RegexReplace('\s{2,}', MapForm_lb_MapType.Items[MapForm_lb_MapType.ItemIndex], ' / ', False);
  GetMapInfo('basic');                   // pocet kachli, mnozstvi dat
  Polygons := TStringList.Create;        // nacte dostupne regiony
  try
    FileList(GEOGET_DATADIR + '\polygon', Polygons);
    MapMngForm_cb_Polygons.Items.Clear;
    for i := 0 to Polygons.Count-1 do if RegexFind('\\$',Polygons[i]) then MapMngForm_cb_Polygons.Items.Add(SeparateLeft(RegexExtract('[^\\]+\\$',Polygons[i]), '\'));
  finally
    Polygons.Free;
  end;  
  MapMngForm.ShowModal;
end;
